import SparkMD5 from 'spark-md5'
import request from '@/api/request'
export const uploadByPieces = (file, option, url) => {
  return new Promise(async (resolve, reject) => {
    // 读取文件的md5
    let fileMD5
    let fileRederInstance = new FileReader()
    fileRederInstance.readAsBinaryString(file)
    fileRederInstance.addEventListener('load', (e) => {
      let fileBolb = e.target.result
      fileMD5 = SparkMD5.hashBinary(fileBolb)
      readChunkMD5(file, fileMD5, option)
    })

    // 针对每个文件进行chunk处理
    const readChunkMD5 = async (file, md5, options) => {
      const CHUNK_SIZE = 5 * 1024 * 1024 // 5MB
      const MAX_RETRIES = 3 // 最大重试次数
      const RETRY_DELAY = 1000 // 重试延迟(ms)

      const chunkCount = Math.ceil(file.size / CHUNK_SIZE)
      const uploadedChunks = [] // 跟踪已上传的分片
      let lastProgress = 0 // 跟踪上次进度

      // 进度更新函数
      const updateProgress = (current) => {
        const progress = Math.round((current / chunkCount) * 100)
        if (progress > lastProgress && options?.onProgress) {
          options.onProgress(progress)
          lastProgress = progress
        }
      }

      for (let i = 0; i < chunkCount; i++) {
        let retries = 0
        let success = false

        while (retries < MAX_RETRIES && !success) {
          try {
            const { chunk } = getChunkInfo(file, i, CHUNK_SIZE)
            await uploadChunk(
              {
                chunk,
                currentChunk: i,
                chunkCount,
                md5
              },
              options
            )

            uploadedChunks.push(i)
            success = true
            updateProgress(i + 1) // 更新进度
          } catch (error) {
            retries++

            if (retries >= MAX_RETRIES) {
              throw new Error(`分片 ${i} 上传失败: ${error.message} (${MAX_RETRIES}次重试后)`)
            }

            // 指数退避重试
            const delay = RETRY_DELAY * Math.pow(2, retries)
            await new Promise((resolve) => setTimeout(resolve, delay))
          }
        }
      }

      return uploadedChunks // 返回成功上传的分片索引
    }

    // const readChunkMD5 = async (file, md5, option) => {
    //   console.log('开始上传',file)
    //   let chunkSize = 5 * 1024 * 1024 // 5MB一片
    //   if(file.size / 1024 / 1024 > 80){
    //   chunkSize = 10 * 1024 * 1024
    //   }

    //   const chunkCount = Math.ceil(file.size / chunkSize) // 总片数
    //   for (var i = 0; i < chunkCount; i++) {
    //     const { chunk } = getChunkInfo(file, i, chunkSize)
    //     await uploadChunk({ chunk, currentChunk: i, chunkCount, md5 }, option)
    //   }
    // }
    const getChunkInfo = (file, currentChunk, chunkSize) => {
      let start = currentChunk * chunkSize
      let end = Math.min(file.size, start + chunkSize)
      let chunk = file.slice(start, end)
      chunk = blobToFile(chunk, file.name)
      return { start, end, chunk }
    }
    // Blob 转 File
    const blobToFile = (blob, fileName) => {
      const file = new File([blob], fileName, { type: blob.type })
      return file
    }
    const uploadChunk = (chunkInfo, option) => {
      // 创建formData对象，下面是结合不同项目给后端传入的对象。
      let formData = new FormData()
      if (Object.keys(option).length > 0) {
        for (let key in option) {
          formData.append(key, option[key])
        }
      }
      formData.append('file', chunkInfo.chunk)
      formData.append('md5', chunkInfo.md5)
      formData.append('chunk_index', chunkInfo.currentChunk)
      formData.append('chunk_total', chunkInfo.chunkCount)
      request
        .post(url, formData)
        .then((res) => {
          if (res.data.src) {
            resolve(res)
          } else if (res.status !== 200) {
            reject(res)
          }
        })
        .catch((e) => {
          reject(e)
        })
    }
  })
}
